# include <stdio.h>
# include <unistd.h>
# include <math.h>
# include <float.h>
# include <limits.h>
# include <sys/time.h>
#include <sys/time.h>
#ifdef _OPENMP
#include <omp.h>
#endif

#define STREAM_ARRAY_SIZE 20000000

#define NTIMES 500

#define STREAM_TYPE double

#define OFFSET 0



static STREAM_TYPE  a[STREAM_ARRAY_SIZE+OFFSET],
                    b[STREAM_ARRAY_SIZE+OFFSET],
                    c[STREAM_ARRAY_SIZE+OFFSET];

extern double mysecond();
extern void checkSTREAMresults();


int
main()
    {
    int quantum;
    int BytesPerWord;
    int k = 1;
    int stride=8;
    ssize_t j;
    STREAM_TYPE scalar;
    double t;
    double tt ;
    tt = mysecond();

    BytesPerWord = sizeof(STREAM_TYPE);

    printf("Array size = %llu (elements), Offset = %d (elements)\n" , (unsigned long long) STREAM_ARRAY_SIZE, OFFSET);
    printf("Memory per array = %.1f MiB (= %.1f GiB).\n",
            BytesPerWord * ( (double) STREAM_ARRAY_SIZE / 1024.0/1024.0),
            BytesPerWord * ( (double) STREAM_ARRAY_SIZE / 1024.0/1024.0/1024.0));
    printf("Total memory required = %.1f MiB (= %.1f GiB).\n",
            (3.0 * BytesPerWord) * ( (double) STREAM_ARRAY_SIZE / 1024.0/1024.),
            (3.0 * BytesPerWord) * ( (double) STREAM_ARRAY_SIZE / 1024.0/1024./1024.));
    printf("Each kernel will be executed %d times.\n", NTIMES);

#pragma omp parallel
    {
#pragma omp master
        {
#ifdef _OPENMP
            k = omp_get_num_threads();
#endif
            printf ("Number of Threads requested = %i\n",k);
        }
    }

#pragma omp parallel for
    for (j=0; j<STREAM_ARRAY_SIZE; j++) {
        a[j] = 1.0;
        b[j] = 2.0;
        c[j] = 0.0;
    }

    t = mysecond();

    /* --- MAIN LOOP --- repeat test cases NTIMES times --- */

    scalar = 3.0;
    for (k=0; k<NTIMES; k++)
    {
#pragma omp parallel for simd
    for (j=0; j<STREAM_ARRAY_SIZE; j+=stride)
        a[j] = b[j]+scalar*c[j];
    }

    /* --- Check Results --- */
    checkSTREAMresults();

    printf("Runtime %12.4f \n", mysecond() - tt);

    return 0;
}


double mysecond()
{
        struct timeval tp;
        struct timezone tzp;
        int i;

        i = gettimeofday(&tp,&tzp);
        return ( (double) tp.tv_sec + (double) tp.tv_usec * 1.e-6 );
}

void checkSTREAMresults ()
{
    ssize_t j;
    STREAM_TYPE sum;
    for (j=0; j<STREAM_ARRAY_SIZE; j++) {
        sum = a[j] + b[j] + c[j];
    }
    printf("sum %f \n", sum);
}

