#include <omp.h>

#include <likwid.h>

void copy(int size, double* dst, double* src)
{
    int j=0;

#ifdef _OPENMP
#pragma omp parallel
{
#endif
    LIKWID_MARKER_START("copy");
#ifdef _OPENMP
#pragma omp for simd
#endif
    for (j=0; j<size; j++)
    {
        dst[j] = src[j];
    }
    LIKWID_MARKER_STOP("copy");
#ifdef _OPENMP
}
#endif
}

void scale(int size, double* dst, double* src, double scalar)
{
    int j=0;
#ifdef _OPENMP
#pragma omp parallel
{
#endif
    LIKWID_MARKER_START("scale");
#ifdef _OPENMP
#pragma omp for simd
#endif
    for (j=0; j<size; j++)
    {
        dst[j] = scalar * src[j];
    }
    LIKWID_MARKER_STOP("scale");
#ifdef _OPENMP
}
#endif
}

void add(int size, double* dst, double* src1, double* src2)
{
    int j=0;
#ifdef _OPENMP
#pragma omp parallel
{
#endif
    LIKWID_MARKER_START("add");
#ifdef _OPENMP
#pragma omp for simd
#endif
    for (j=0; j<size; j++)
    {
        dst[j] = src1[j] + src2[j];
    }
    LIKWID_MARKER_STOP("add");
#ifdef _OPENMP
}
#endif
}

void triad(int size, double* dst, double* src1, double* src2, double* src3)
{
    int j=0;
#ifdef _OPENMP
#pragma omp parallel
{
#endif
    LIKWID_MARKER_START("triad");
#ifdef _OPENMP
#pragma omp for simd
#endif
    for (j=0; j<size; j++)
    {
        dst[j] = (src1[j] * src2[j]) + src3[j];
    }
    LIKWID_MARKER_STOP("triad");
#ifdef _OPENMP
}
#endif
}
